# -*- coding: iso-8859-1 -*-
from __future__ import division # 1/2 == .5 (par defaut, 1/2 == 0)
from __future__ import with_statement

#    WxGeometrie
#    Dynamic geometry, graph plotter, and more for french mathematic teachers.
#    Copyright (C) 2005-2007  Nicolas Pourcelot
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

# version unicode


from LIB import *
import matplotlib


# uncomment the following to use wx rather than wxagg
# matplotlib.use('WX')
# from matplotlib.backends.backend_wx import FigureCanvasWx as FigureCanvas
from matplotlib.figure import Figure
from matplotlib.backends.backend_wxagg import FigureCanvasWxAgg as FigureCanvas
from matplotlib.backends.backend_wxagg import FigureCanvasAgg
from matplotlib.backends.backend_agg import RendererAgg
from matplotlib._image import frombyte








class Moteur_graphique(object):
    def __init__(self, parent):
        self.parent = parent
##        # lments en attente d'tre dessins
##        self._queue = []
        self._dernier_objet_deplace = None
        self.axes = self.parent.axes
        # Lgre optimisation :
        # les objets sont copis et modifis plutt que recrs
        self.__empty_plot = self.axes.plot([0,1], [1,0])[0]
        self.__empty_fill = self.axes.fill([0, 1, 1, 0], [1, 0, 0, 1])[0]
        self.__empty_text = self.axes.text(0, 0, '')
        self.__empty_text._fontproperties.set_family('serif')
        self.__empty_arrow = self.axes.arrow(0, 0, 1, 1)
        # Buffer contenant la dernire image.
        self._dernier_dessin = None
##        self.mise_en_cache_axes = [[],[], [], []] # plot, fill, text, artist
        #if self.parent.transformation is not None:
            #self.__empty_plot.set_transform(self.parent.transformation)
            #self.__empty_fill.set_transform(self.parent.transformation)
            #self.__empty_text.set_transform(self.parent.transformation)

    # -------------------
    # Fonctions de dessin
    # -------------------


    def _ajouter_objet(self, artiste):
        self.axes.artists.append(artiste)

    def _ajouter_objets(self, liste_artistes):
        self.axes.artists.extend(liste_artistes)

    def ajouter(self, artiste):
        u"""Ajoute un artiste (objet graphique de matplotlib)  dessiner.

        NB: 'artiste' peut aussi tre une liste d'artistes, ou une liste de listes..."""
        if isinstance(artiste, (list, tuple)):
            for elt in artiste:
                self.ajouter(elt)
        else:
            self._ajouter_objet(artiste)


    def ligne(self, x = None, y = None, couleur = None, pixel = False, **kw):
        ligne = module_copy.copy(self.__empty_plot)
        if x is not None and y is not None:
            if pixel:
                x, y = self.parent.pix2coo(x, y)
            if self.parent.transformation is not None:
                x, y = zip(*(zip(x, y)*self.parent.transformation).array)
            ligne.set_data(x, y)
        if couleur is not None:
            ligne.set_color(couleur)
        if kw:
            ligne.set(**kw)
        return ligne

    def ajouter_ligne(self, x = None, y = None, couleur = None, pixel = False, **kw):
        self._ajouter_objet(self.ligne(x, y, couleur, pixel, **kw))


    def polygone(self, x = None, y = None, couleur = None, pixel = False, **kw):
        polygone = module_copy.copy(self.__empty_fill)
        if x is not None and y is not None:
            if pixel:
                x, y = self.parent.pix2coo(x, y)
            if self.parent.transformation is not None:
                x, y = zip(*(zip(x, y)*self.parent.transformation).array)
            polygone.xy = zip(x, y)
        if couleur is not None:
            polygone.set_facecolor(couleur)
        if kw:
            polygone.set(**kw)
        return polygone

    def ajouter_polygone(self, x = None, y = None, couleur = None, pixel = False, **kw):
        self._ajouter_objet(self.polygone(x, y, couleur, pixel, **kw))


    def texte(self, x = None, y = None, txt = None, pixel = False, **kw):
        texte = module_copy.copy(self.__empty_text)
        if x is not None and y is not None:
            if pixel:
                x, y = self.parent.pix2coo(x, y)
            if self.parent.transformation is not None:
                x, y = ((x, y)*self.parent.transformation).array[0]
            texte.set_x(x)
            texte.set_y(y)
        if txt is not None:
            if param.unicode and not isinstance(txt, unicode):
                txt = unicode(txt, param.encodage)
            texte.set_text(txt)
        if kw:
            texte.set(**kw)
        return texte

    def ajouter_texte(self, x = None, y = None, txt = None, pixel = False, **kw):
        self._ajouter_objet(self.texte(x, y, txt, pixel, **kw))


    def arc(self, x, y, vecteur, couleur = "k", **kw):
        # affiche un petit demi-cercle d'orientation choisie
        # Par ex, pour le vecteur (1,0), ie. ->, l'arc est orient comme ci: (
        # Pour le vecteur (-1,0), ie. <-, l'arc est orient comme ci: ).
        # On travaille sur les pixels pour faire de la trigonometrie plus simplement
        # (le repere devient ainsi orthonorme)
        vecteur = self.parent.dcoo2pix(*vecteur)
        if vecteur[0] == 0:
            if vecteur[1] > 0:
                angle = math.pi/2
            else:
                angle = -math.pi/2
        else:
            angle = math.atan(vecteur[1]/vecteur[0])
            if vecteur[0] < 0:
                angle += math.pi
        # donne l'angle d'incidence  l'extrmit
        t = numpy.arange(angle - math.pi/2, angle + math.pi/2, 0.05)
        R = self.parent.taille["("]*param.zoom_ligne

        x, y = self.parent.coo2pix(x, y)
        return self.ligne(x + R*(numpy.cos(t) - math.cos(angle)), \
                                     y + R*(numpy.sin(t) - math.sin(angle)), \
                                     couleur = couleur, pixel = True, **kw)

    def ajouter_arc(self, x, y, vecteur, couleur = "k", **kw):
        self._ajouter_objet(self.arc(x, y, vecteur, couleur, **kw))



    def point(self, x, y, couleur = 'k', plein = True, **kw):
        kw.setdefault('zorder', 2.1)
        kw.setdefault('markeredgecolor', couleur)
        kw.setdefault('markeredgewidth', 1)
        return self.ligne([x], [y], couleur if plein else 'w', \
                      markersize = 2*self.parent.taille["o"], marker = "o", **kw)
        # Le zorder doit tre suprieur  celui d'une courbe (2 par dfaut),
        # et la largeur du trait doit tre celle d'une courbe (1 par dfaut).

    def ajouter_point(self, x, y, couleur = 'k', plein = True, **kw):
        self._ajouter_objet(self.point(x, y, couleur, plein, **kw))


    def fleche(self, x0, y0, x1, y1, **kw):
        arrow = module_copy.copy(self.__empty_arrow)
        #TODO: continuer code (matplotlib 0.99+ requis)
        # cf. matplotlib.patches.ArrowStyle.get_styles() pour les
        # styles de flches.
        return arrow

    def ajouter_fleche(self, x0, y0, x1, y1, **kw):
        self._ajouter_objet(self.fleche(self, x0, y0, x1, y1, **kw))


    # ----------------------
    # Gestion de l'affichage
    # ----------------------



    def _creer_arriere_plan(self):
        if self.parent.rafraichir_axes:
            dict_artistes = self._effacer_artistes()
            self._dessine_axes()
            FigureCanvasAgg.draw(self.parent)
##            self.parent.draw(repaint = False)
##            self._dessiner_artistes()
            self._mise_en_cache_axes = self._en_cache()
            self._restaurer_artistes(dict_artistes)
            self.parent.rafraichir_axes = False
        else:
            self._restaurer(self._mise_en_cache_axes)


    def _regler_fenetre(self):
        xmin, xmax, ymin, ymax = self.parent.fenetre
        if param.old_matplotlib:
            self.axes.viewLim.intervalx().set_bounds(xmin, xmax)
            self.axes.viewLim.intervaly().set_bounds(ymin, ymax)
        else:
            self.axes.viewLim.set_points(pylab.array([[xmin, ymin], [xmax, ymax]]))


    def dessiner(self, dessin_temporaire = False, objet_deplace = None):
##        print "Dessin."
        # Affichage bloqu
        if self.parent.affichage_gele:
##            print u"gel"
            return
##        print u"pas gel"
        # Les axes et lgendes de matplotlib ne conviennent pas, on les dsactive.
        self.axes.legend_ = None
        self.axes.axison = False
        # Pas de feuille
        if self.parent.feuille_actuelle is None:
            self._effacer_artistes()
            self.parent.figure.set_facecolor(self.parent.couleur_fond_vide)
            self.parent.draw()
            self._dernier_dessin = self._en_cache()
            self.parent.figure.set_facecolor(self.parent.couleur_fond)
            return
        # Cas normal
        self._regler_fenetre()
        if dessin_temporaire:
            self._restaurer(self._dernier_dessin)
        else:
            if self.parent.rafraichir_axes:
                self.parent.feuille_actuelle.rafraichir_figures()
            if objet_deplace is None or self.parent.rafraichir_axes:
                # Cas par dfaut : on dessine tout.
                self._creer_arriere_plan()
                self._objets_fixes, self._objets_mobiles = self.parent.feuille_actuelle.lister_figures(objet_deplace)
                self._ajouter_objets(self._objets_fixes)
            elif objet_deplace is self._dernier_objet_deplace:
                # Un cache spcifique existe dj : on l'utilise.
##                print "Optimisation."
                self._restaurer(self._mise_en_cache_objets_fixes)
            else:
                # On gnre un cache (spcifique  l'objet dplac).
                self._creer_arriere_plan()
                dict_artistes = self._effacer_artistes()
                self._objets_fixes, self._objets_mobiles = self.parent.feuille_actuelle.lister_figures(objet_deplace)
                self._ajouter_objets(self._objets_fixes)
                self._dessiner_artistes()
                self._mise_en_cache_objets_fixes = self._en_cache()
                self._restaurer_artistes(dict_artistes)
            self._ajouter_objets(self._objets_mobiles)
            # Rsum du bloc de code prcdent :
            # Les objets fixes ne doivent pas tre redessins,
            # mais restitus  partir d'un cache.
            # On gnre le cache la premire fois que objet_deplace is not None
            # et on utilise le cache les autres fois si objet_deplace
            # correspond toujours au mme objet.

        # On dessine dans le buffer
        self._dessiner_artistes()
        # Affichage proprement dit (copie du buffer  l'cran)
        self.parent.blit()
        # Garde en mmoire l'affichage pour viter de redessiner la fentre
        # dans certains cas (fentre masque, etc.)
        if not dessin_temporaire:
            self._dernier_dessin = self._en_cache()
        self._dernier_objet_deplace = objet_deplace
        self._effacer_artistes()


    def restaurer_dessin(self):
        u'''Restaure le dernier dessin.'''
        self._restaurer(self._dernier_dessin)
        self.parent.blit()


    def _effacer_artistes(self):
        u"Supprime tous les artistes (objets graphiques de matplotlib)."
        dico = {}
        for rubrique in ("lines", "patches", "texts", "tables", "artists", "images", "collections"):
            dico[rubrique] = getattr(self.axes, rubrique)
        self.axes.artists = []
        self.axes.lines = []
        self.axes.patches = []
        self.axes.texts = []
        self.axes.tables = []
        self.axes.images = []
        self.axes.collections = []  # collection.Collection instances
        return dico


    def _restaurer_artistes(self, dictionnaire):
        u"Restaure les artistes prcdemment supprims."
        self.axes.__dict__.update(dictionnaire)


    def _dessiner_artistes(self):
        artistes = list(self.axes.artists)
        artistes.extend(self.axes.lines)
        artistes.extend(self.axes.patches)
        artistes.extend(self.axes.texts)
        artistes.extend(self.axes.tables)
        artistes.extend(self.axes.collections)
        artistes.sort(key = lambda x: x.zorder)
        regler_textes = abs(param.zoom_texte - 1) > 0.05
        regler_lignes = abs(param.zoom_ligne - 1) > 0.05
        # C'est ici qu'a lieu le dessin proprement dit.
##        print "artistes:", len(artistes), artistes
        for artiste in artistes:
            if artiste._visible:
                try:
                    if regler_textes:
                        if isinstance(artiste, matplotlib.text.Text):
                            artiste._size_init_ = artiste.get_size()
                            artiste.set_size(artiste._size_init_*param.zoom_texte)
                    if regler_lignes:
                        if isinstance(artiste, matplotlib.lines.Line2D):
                            artiste._linewidth_init_ = artiste.get_linewidth()
                            artiste._markersize_init_ = artiste.get_markersize()
                            artiste._markeredgewidth_init_ = artiste.get_markeredgewidth()
                            artiste.set_linewidth(artiste._linewidth_init_*param.zoom_ligne)
                            artiste.set_markersize(artiste._markersize_init_*param.zoom_ligne)
                            artiste.set_markeredgewidth(artiste._markeredgewidth_init_*param.zoom_ligne)
                    self.axes.draw_artist(artiste)
                finally:
                    if regler_textes:
                        if isinstance(artiste, matplotlib.text.Text):
                            artiste.set_size(artiste._size_init_)
                    if regler_lignes:
                        if isinstance(artiste, matplotlib.lines.Line2D):
                            artiste.set_linewidth(artiste._linewidth_init_)
                            artiste.set_markersize(artiste._markersize_init_)
                            artiste.set_markeredgewidth(artiste._markeredgewidth_init_)

    def _en_cache(self):
        return self.parent.copy_from_bbox(self.axes.bbox)

    def _restaurer(self, buffer):
        if buffer is not None:
            self.parent.restore_region(buffer)


##    def ajouter(self, *artistes):
##        self._queue.extend(artistes)



    def _dessine_axe(self, num = 0, nbr_axes = 1):
        u"Dessine l'axe des abscisses (num = 0) ou des ordonnes (num = 1)."

        #un = nbr_axes == 1              # un seul axe
        #x = num == 0; y = not x         # axe des abscisses / des ordonnees
        #rep = self.utiliser_repere      # repere ou non
        vect = self.parent.utiliser_repere and self.parent.repere[1 + num] in string.ascii_lowercase
        # repere defini ou non par des vecteurs
        correspondance = {"b" : "bottom", "c": "center", "l": "left", "r": "right", "t": "top"}

        def f(a, b):
            return a if num == 0 else b

        signe = f(-1, 1)
        origine = self.parent.origine_axes[1 - num]
        fenetre = self.parent.fenetre
        repere = self.parent.repere

        # On cree l'axe proprement dit:
        self.ajouter_ligne(f([fenetre[0], fenetre[1]], [origine, origine]), f([origine, origine], [fenetre[2], fenetre[3]]), "k")

        if self.parent.affiche_fleches:
            # On cree la fleche au bout de l'axe:
            x, y = self.parent.coo2pix(f(fenetre[1], origine), f(origine, fenetre[3]))
            dx, dy = .5*self.parent.taille[">"]*param.zoom_ligne*pylab.array([f(math.sqrt(3), 1), f(1, math.sqrt(3))])
            self.ajouter_ligne([x - dx, x, x + signe*dx], [y + dy, y, y + signe*dy], "k",  pixel = True)


        # FORMAT (_origine_ et _graduation_) : [texte, x, y, alignement horizontal, alignement vertical]
        # _origine_ : infos sur l'origine de l'axe
        # _graduation_ : infos sur le 2eme point de l'axe

        ox, oy = self.parent.origine_axes
        ux, uy = self.parent.gradu
        # TODO:  tendre  un repre non orthogonal
        kx, ky = self.parent.dpix2coo(1, 1)

        x, y = self.parent.coo2pix(f(ox + ux, ox), f(oy, oy + uy))
        if self.parent.gradu[num] and self.parent.saturation(num) < param.saturation:
            if vect:
                self.ajouter_ligne([x - dx, x, x + signe*dx], [y + dy, y, y + signe*dy], "k", pixel = True)
            elif self.parent.affiche_quadrillage:
                d = .5*self.parent.taille["|"]*param.zoom_ligne
                self.ajouter_ligne([f(x, x - d), f(x, x + d)], [f(y - d, y), f(y + d, y)], "k", pixel = True)


        if nbr_axes == 1:                   # un seul axe
            if num:                         # axe des ordonnees
                if self.parent.utiliser_repere:    # utiliser un repere (meme origine sur chaque axe...)
                    _origine_ = [repere[0], ox - 8*kx, oy, "r", "c"]
                    if vect:                # reperage par des vecteurs
                        _graduation_ = [u"$\\tt{\\vec{" + repere[2] + "}}$", ox - 8*kx, oy + .5*uy, "r", "c"]
                    else:                   # reperage par des points
                        _graduation_ = [repere[2], ox - 8*kx, oy + uy, "r", "c"]
                else:                       # pas un repere
                        _origine_ = [str(oy), ox - 8*kx, oy, "r", "c"]
                        _graduation_ = [str(oy + uy), ox - 8*kx, oy + uy, "r", "c"]
            else:                           # axe des abscisses
                if self.parent.utiliser_repere:    # utiliser un repere (meme origine sur chaque axe...)
                    _origine_ = [repere[0], ox, oy - 8*ky, "c", "t"]
                    if vect:                # reperage par des vecteurs
                        _graduation_ = [u"$\\tt{\\vec{" + repere[1] + "}}$", ox + .5*ux, oy - 8*ky, "c", "t"]
                    else:                   # reperage par des points
                        _graduation_ = [repere[1], ox + ux, oy - 8*ky, "c", "t"]
                else:                       # pas un repere
                        _origine_ = [str(ox), ox, oy - 8*ky, "c", "t"]
                        _graduation_ = [str(ox + ux), ox + ux, oy - 8*ky, "c", "t"]
        else:                               # 2 axes
            if num:                         # axe des ordonnees
                if self.parent.utiliser_repere:
                    _origine_ = None
                    if vect:                # reperage par des vecteurs
                        _graduation_ = ["\\vec{" + repere[2] + "}", ox - 8*kx, oy + .5*uy, "r", "c"]
                    else:                   # reperage par des points
                        _graduation_ = [repere[2], ox - 8*kx, oy + uy, "r", "c"]
                else:                       # pas un repere
                        _origine_ = [str(oy), ox - 8*kx, oy + 3*ky, "r", "b"]
                        _graduation_ = [str(oy + uy), ox - 8*kx, oy + uy, "r", "b"]
            else:                           # axe des abscisses
                if self.parent.utiliser_repere:
                    _origine_ = [repere[0], ox - 8*kx, oy - 8*ky, "r", "t"]
                    if vect:                # reperage par des vecteurs
                        _graduation_ = ["\\vec{" + repere[1] + "}", ox + .5*ux, oy - 8*ky, "c", "t"]
                    else:                   # reperage par des points
                        _graduation_ = [repere[1], ox + ux, oy - 8*ky, "c", "t"]
                else:                       # pas un repere
                        _origine_ = [str(ox), ox + 3*kx, oy - 8*ky, "l", "t"]
                        _graduation_ = [str(ox + ux), ox + ux, oy - 8*ky, "l", "t"]

        if _origine_ and self.parent.gradu[num]:
            self.ajouter_texte(_origine_[1], _origine_[2], "$" + _origine_[0] + "$", \
                horizontalalignment = correspondance[_origine_[3]], \
                verticalalignment = correspondance[_origine_[4]], size = 14*param.zoom_texte)

        if _graduation_ and self.parent.gradu[num] and self.parent.saturation(num) < param.saturation:
            self.ajouter_texte(_graduation_[1], _graduation_[2], "$" + _graduation_[0] + "$", \
                horizontalalignment = correspondance[_graduation_[3]], \
                verticalalignment = correspondance[_graduation_[4]], size = 14*param.zoom_texte)






    def _quadriller(self, hauteur = 3, pas = (None, None), style = None, epaisseur = 1, couleur = "k"):
        u"""Cre des graduations ou un quadrillage (si hauteur = None).
        Si pas[0] = 0 (respectivement, pas[1] = 0), les graduations seront
        uniquement horizontales (respectivement verticales)."""

        args = []
        origine = self.parent.origine_axes
        fenetre = self.parent.fenetre
        pas = list(pas)
        # Je n'utilise pas des listes directement, mais des tuples.
        # Ceci afin de n'avoir jamais la MEME liste.

        if hauteur == None: # quadrillage
            xmin, xmax, ymin, ymax = fenetre
            _min = ymin, xmin ; _max = ymax, xmax
            if not style:
                style = ":"
        else:               # graduations
##            _min = [origine[i] - hauteur*self.parent.coeff(i) for i in (1, 0)] # ymin, xmin
##            _max = [origine[i] + hauteur*self.parent.coeff(i) for i in (1, 0)] # ymax, xmax
            # ymin, xmin:
            _min = [origine[1] - self.parent.dpix2coo(0, hauteur)[1],
                    origine[0] - self.parent.dpix2coo(hauteur, 0)[0]]
            # ymax, xmax:
            _max = [origine[1] + self.parent.dpix2coo(0, hauteur)[1],
                    origine[0] + self.parent.dpix2coo(hauteur, 0)[0]]
            if not style:
                style = "-"

##        #type_ligne = couleur + style

        # Format de 'segments': [[(x0,y0), (x1,y1), ...], [...], [...], ...]
        # cf. matplotlib.collections.LineCollection
        segments = []

        for n in (0, 1):
            if pas[n] == None:  pas[n] = self.parent.gradu[n]
            pas[n] = 1.*abs(pas[n])

            # graduations sur l'axe:
            if pas[n] and self.parent.coeff(n)/pas[n] < param.saturation:

                maxi = fenetre[2*n + 1]
                if self.parent.affiche_axes and (n in self.parent.liste_axes) and hauteur:
                    # les graduations qui chevauchent les fleches, c'est pas joli
                    maxi -= .7*self.parent.taille[">"]*param.zoom_ligne*math.sqrt(3)*self.parent.coeff(n)

                valeurs = pylab.concatenate((pylab.arange(origine[n], fenetre[2*n], -pas[n]), pylab.arange(origine[n] + pas[n], maxi, pas[n])))
                for val in valeurs:
                    # si on affiche le quadrillage ou les graduations sur les axes, ca fait pas net:
                    if val != origine[n] or not self.parent.affiche_axes or not (1 - n in self.parent.liste_axes):
                        if n:
                            segments.append([(_min[n], val), (_max[n], val)])
                        else:
                            segments.append([(val, _min[n]), (val, _max[n])])


##        m = len(segments)
##        width = m*[epaisseur]
##        color = m*[matplotlib.colors.colorConverter.to_rgba(couleur)]
        couleur = matplotlib.colors.colorConverter.to_rgba(couleur)
        style_dict = {':': 'dotted', '--': 'dashed', '-.': 'dashdot', '-': 'solid'}
        style = style_dict.get(style, style)
        collection = matplotlib.collections.LineCollection(segments, linewidths = epaisseur, colors = couleur,  linestyle = style)
        self.axes.add_collection(collection)




    def _dessine_axes(self):
            if self.parent.affiche_axes:
                for axe in self.parent.liste_axes:
                    self._dessine_axe(axe, len(self.parent.liste_axes))

            if self.parent.affiche_quadrillage:   # quadrillage
                for quadrillage in self.parent.quadrillages:
                    self._quadriller(None, *quadrillage)
            elif self.parent.affiche_axes:
                self._quadriller()   # simples graduations


