# -*- coding: iso-8859-1 -*-
from __future__ import division # 1/2 == .5 (par defaut, 1/2 == 0)
from __future__ import with_statement

##--------------------------------------#######
#                  Feuille                                  #
##--------------------------------------#######
#    WxGeometrie
#    Dynamic geometry, graph plotter, and more for french mathematic teachers.
#    Copyright (C) 2005-2009  Nicolas Pourcelot
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA



# Ce module contient essentiellement la classe Feuille. C'est la classe qui va accueillir tous les objets geometriques.
# Elle fait l'intermediaire entre les objets et le Panel ou s'affichent les objets.

import keyword

from ALL import *
import ALL
from pseudo_canvas import _pseudocanvas
import math
import mathlib
import sympy

assert ALL.Objet is Objet
##class Dict_objets(dict):
##    u"""Utilis pour l'instruction exec."""
##    def __init__(self, objets, dico = None):
##        if dico is None:
##            dict.__init__(self)
##        else:
##            dict.__init__(self, dico)
##        self.objets = objets
##
##    def __setitem__(self, key, val):
##        setattr(self.objets, key, val)
##
##    def __delitem__(self, key):
##        delattr(self.objets, key)


class Dictionnaire_objets(dict):
    u"""Cette classe est un conteneur pour les objets de la feuille, qui sont tous ses attributs,
sauf ceux prcds de __ (attributs reserves pour tous les objets de la classe).
Elle contient aussi tous les objets geometriques.
Certaines methodes standard (comme __setattr__) sont aussi interceptees ou redefinies.

Attributs spciaux:
'_' fait rfrence au dernier objet enregistr dans la feuille.
"""

    __slots__ = ("__feuille__", "__timestamp")

    def __init__(self, feuille):
        object.__setattr__(self, "__feuille__", feuille)
        object.__setattr__(self, "_Dictionnaire_objets__timestamp", 0)
        self.__effacer()


    def __effacer(self):
        self.clear()
        # On ajoute au dictionnaire courant les objets gomtriques, et uniquement eux
        # (pas tout LIB.py !)
        self.update((key, val) for key, val in ALL.__dict__.iteritems() \
                    if isinstance(val, type) and issubclass(val, ALL.Objet))
        # Les noms de classe peuvent aussi tre taps en minuscules (c'est plus rapide  taper)
        self.update((key.lower(), val) for key, val in ALL.__dict__.iteritems() \
                    if isinstance(val, type) and issubclass(val, ALL.Objet))

        # On ajoute au dictionnaire les fonctions mathmatiques courantes
        self.update((key, val) for key, val in mathlib.universal_functions.__dict__.iteritems() \
                    if key[0] != "_" and key != "division")

        self.update(pi = math.pi, e = math.e, oo = sympy.oo, \
                    Intervalle = intervalles.Intervalle, Union = intervalles.Union, \
                    x = sympy.Symbol("x"), y = sympy.Symbol("y"), z = sympy.Symbol("z"), \
                    t = sympy.Symbol("t"))


        self.update(pause = self.__feuille__.pause, erreur = self.__feuille__.erreur,
                    effacer = self.__feuille__.effacer, True = True, False = False,
                    coder = self.__feuille__.coder, effacer_codage = self.__feuille__.effacer_codage,
                    )
        # NB: 'True = True' et 'False = False' : non compatible Py3k


    def __setattr__(self,nom,val):
        return self.__setitem__(nom, val)
    def __getattr__(self,nom):
        return self.__getitem__(nom)
    def __delattr__(self,nom):
        return self.__delitem__(nom)


    def add(self, valeur):
        self["_"] = valeur

##    def _remove(self, valeur):
##        u"Commande de bas niveau. Ne pas utiliser directement !"
##        for (key, val) in self.items():
##            if val is valeur:
##                self.pop(key)

    def _dereferencer(self, objet):
        u"Commande de bas niveau. Ne pas utiliser directement !"
        if objet._nom:
            self.pop(objet._nom)
            # Important: pour que l'objet soit bien considr non rfrenc
            # il faut qu'il n'ait pas de nom (on ne peut pas rfrencer 2 fois un objet).
            objet._nom = ""

    def __setitem__(self, nom, valeur):
        u"""Note: objets.__setattr__("a", 3) <-> objets.a = 3
        Les objets d'une feuille, contrairement aux objets Python, ne peuvent pas tre redfinis comme ca...
        En gnral, quand on essaie d'attribuer un nom qui existe dj, ce n'est pas volontaire.
        Pour viter les erreurs, on impose de d&truire explicitement l'objet avant que le nom puisse tre redonn."""


##        # Stocke le rsultat de la dernire commande (objet gomtrique ou non)
##        if nom == '__':
##            self.__feuille__.__ = valeur
##            return

        # Modifie la fentre d'affichage
        if nom == "fenetre":
            self.__feuille__.fenetre = valeur
            return

        nom = self.__verifier_syntaxe_nom(valeur, nom)

        # Pour certains types d'objets (points libres, textes, variables...),
        # le nom peut tre dja attribu.
        # Par exemple, A=Point(1,2) est valide mme si A existe dj.
        # L'objet A ne sera pas cras, mais actualis.
        # Dans cet exemple, les coordonnes de A sont actualises en (1,2)
        # (Autrement dit, A=(1,2) devient un alias de A(1,2) ou A.coordonnees = 1,2).
        # Bien sr, il faut en particulier que la valeur soit un objet de meme type.
        # (A = Variable(3) n'est pas valide si A est un point !)
##        if self.has_key(nom):
##            if hasattr(self[nom], "_update"):
##                self[nom]._update(valeur)
##                return # on quitte, car le nom doit toujours rfrer  l'objet initial !
##            else:
##                self.erreur(u"ce nom est deja utilis\xe9 : " + nom, NameError)



        if self.has_key(nom):
            try:
                # Pour que les variables puissent tre interprtes, il faut que la feuille soit donne
                if isinstance(valeur, Objet) and valeur.__feuille__ is None:
                    valeur.__feuille__ = self.__feuille__
                self[nom]._update(valeur)
                return # on quitte, car le nom doit toujours rfrer  l'objet initial !
            except:
                print_error()
                self.erreur(u"Ce nom est d\xe9ja utilis\xe9 : " + nom, NameError)



        if not isinstance(valeur, Objet):

            # Permet de construire des points  la vole : '=((i,sqrt(i)) for i in (3,4,5,6))'
            if isinstance(valeur, types.GeneratorType) and nom == "_":
                for item in valeur:
                    self.__setitem__('', item)
                return

            # 'A = Point' est un alias de 'A = Point()'
            elif isinstance(valeur, types.TypeType) and issubclass(valeur, Objet):
                valeur = valeur()

            # Par convniance, certains types sont automatiquement convertis :
            # - Variable
            elif isinstance(valeur, (int, long, float, str, unicode)): # u=3 cree une variable
                valeur = Variable(valeur)

            # - Point
            elif isinstance(valeur, complex):
                valeur = Point(valeur.real, valeur.imag)

            elif hasattr(valeur, "__iter__"):
                valeur = tuple(valeur)
                # - Texte
                if len(valeur) in (1, 3) and isinstance(tuple(valeur)[0], basestring):
                    # t=["Bonjour!"] cree un texte
                    valeur = Texte(*valeur)
                elif len(valeur) == 2:
                    # - Vecteur_libre
                    if nom.islower():
                        # u=(1,2) cre un vecteur
                        valeur = Vecteur_libre(*valeur)
                    # - Point
                    else:
                        # A=(1,2) cree un point.
                        valeur = Point(*valeur)

        if not isinstance(valeur, Objet):
            self.erreur("type d'objet incorrect :(%s,%s)"%(nom, valeur), TypeError)

        if valeur._nom:
            # L'objet est dj rfrenc sur la feuille ssi il a un nom.
            # On en fait alors une copie : ainsi, A = B est remplac par A = B.copy()
            valeur = valeur.copy()


        # On enregistre le nom (ventuellement provisoire) car la mthode '_set_feuille' de l'objet en a besoin.
        valeur._nom = nom
        valeur.__feuille__ = self.__feuille__

        if nom == "_":
            # Attention, la feuille doit tre dj definie !
            nom = valeur._nom_alea()
            # Pour les objets nomms automatiquement, le nom n'est pas affich par dfaut.
            if valeur.style("legende") == wxgeo.NOM:
                valeur.style(legende = wxgeo.RIEN)

        # les objets commencant par "_" ne sont pas affichs par dfaut (pure convention) :
        if nom[0] == "_":
            valeur.style(visible = False)

##        if is_in(valeur, self.itervalues()):
##            # si l'objet est deja reference sur la feuille, on en fait une copie.
##            valeur = valeur.copy() # ainsi, A = B est remplace par A = B.copy()

        dict.__setitem__(self, nom, valeur)
##        dict.__setattr__(self,  "__derniere_valeur__", weakref.ref(valeur))
        valeur._nom = nom
        valeur._timestamp = self.__timestamp
        object.__setattr__(self, "_Dictionnaire_objets__timestamp", self.__timestamp + 1)
        valeur.creer_figure(True)
        self.__feuille__._actualiser_liste_objets = True


    def __getitem__(self, nom):
        if nom == "fenetre":
            return self.__feuille__.fenetre
        elif nom == "objets":
            return self()
        elif nom == "noms":
            return self.noms
        elif nom == "_":
            return self.__derniere_valeur()
        try:
            return dict.__getitem__(self, self.__convertir_nom(nom))
        except KeyError:
            self.erreur(u"Objet introuvable sur la feuille : " + nom, KeyError)


    def __delitem__(self, nom):
        if nom == "_":
            self.__derniere_valeur().supprimer()
        else:
            self[nom].supprimer()
        self.__feuille__._actualiser_liste_objets = True


    def lister(self, objets_caches = True, **kw):
        u"""Retourne la liste des objets gomtriques.
        Le paramtre objets_caches indique s'il faut retourner les objets cachs.
        kw:
        'type' : types  inclure
        'sauf' : types  exclure
        """
        # Note: objets.__call__() <-> objets()
        if kw:
            sauf = kw.get("sauf", ())
            type = kw.get("type", Objet)
            return [obj for obj in self.values() if isinstance(obj, type) \
                    and not isinstance(obj, sauf) and (objets_caches or obj.style("visible"))]
        elif objets_caches:
            return [obj for obj in self.values() if isinstance(obj, Objet)]
        else:
            return [obj for obj in self.values() if isinstance(obj, Objet) and obj.style("visible")]


    @property
    def noms(self, objets_caches = True, **kw):
        u"""Retourne les noms de tous les objets gomtriques.
        Le paramtre objets_caches indique s'il faut retourner les objets cachs.
        kw:
        'type' : types  inclure
        'sauf' : types  exclure
        """
        if kw:
            sauf = kw.get("sauf", ())
            type = kw.get("type", Objet)
            return [nom for nom, obj in self.items() if isinstance(obj, type) \
                    and not isinstance(obj, sauf) and (objets_caches or obj.style("visible"))]
        else:
            return [nom for nom, obj in self.items() if isinstance(obj, Objet) \
                    and (objets_caches or obj.style("visible"))]




    @staticmethod
    def __convertir_nom(nom):
        u'''Convertit les noms contenant des `, ', ou " en noms python corrects.'''
        return nom.replace('`', '_prime').replace('"', '_prime_prime').replace("'", "_prime")

    def __verifier_syntaxe_nom(self, objet, nom):
        u"Vrifie que le nom est correct (ie. bien form) et le modifie le cas chant."
        if nom == "":
            return "_"
        # Noms rservs en python (if, then, else, for, etc.):
        if keyword.iskeyword(nom):
            self.erreur(u"Nom r\xe9serv\xe9 : " + nom, NameError)
        # Les noms contenant '__' sont des noms rservs pour un usage futur ventuel (convention).
        if "__" in nom:
            self.erreur(u'Un nom ne peut pas contenir "__".', SyntaxError)
        # Vrifie que le nom n'est pas rserv.
        if nom in self.__class__.__dict__.keys():
            self.erreur(u"Nom rserv.", NameError)
        if not re.match("""[A-Za-z_][A-Za-z0-9_'"`]*$""", nom):
            self.erreur(u"'%s' n'est pas un nom d'objet valide." %nom, SyntaxError)

        # Certains noms sont rservs  des usages spcifiques.
        # Les noms f1, f2... sont rservs aux fonctions (cf. module Traceur).
        if nom[0] == 'f' and nom[1:].isdigit() and not isinstance(objet, Fonction):
            self.erreur(u"Nom r\xe9serv\xe9 aux fonctions : " + nom, NameError)

        # Les noms Cf1, Cf2... sont rservs  l'usage du module Traceur.
        if nom.startswith('Cf') and nom[2:].isdigit() and \
                not(isinstance(objet, Objet) and objet.style('protege')):
            self.erreur(u"Nom r\xe9serv\xe9 : " + nom, NameError)
        return self.__convertir_nom(nom)


    def _objet_renommable(self, objet, nom):
        u"Vrifie que le nom peut-tre attribu (c--d. qu'il est bien form, et non utilis)."
        nom = self.__verifier_syntaxe_nom(objet, nom)
        if self.has_key(nom):
            self.erreur(u"Ce nom est dj utilis.", NameError)
        return nom


    def __str__(self):
        return "Gestionnaire d'objets de la feuille '" + self.__feuille__.nom \
                    + "': " + str(self.noms)

    def __repr__(self):
        return "Gestionnaire d'objets de la feuille '" + self.__feuille__.nom \
                    + "': " + repr(self.noms)

##    def effacer(self):
##        for key, obj in self.items():
##            if isinstance(obj, Objet):
##                try:
##                    del self[key]
##                except KeyError:
##                    pass # la cl a dj t efface (par exemple, il s'agit d'un segment dont une des extrmits a dj t supprime)

    def __derniere_valeur(self):
        u"Dernier objet cr."
        return max(self.__feuille__.liste_objets(True), key = lambda obj:obj._timestamp)



class Interprete_feuille(object):
    u"""Excute des commandes dans la feuille.

    Reformule galement les commandes avant de les excuter."""

    def __init__(self, feuille):
        self.feuille = feuille

    def executer(self, commande, parser = True):
        if parser:
            commande = self.parser(commande)

##        # Commandes commenant par '='
##        if len(commande) > 1 and commande[0] == "=" and commande[1] != "=":
##            commande = "_" + commande

##        if pylib.securite.expression_affectable(commande):
##            commande = "__ = " + commande
##        else:
##            self.feuille.__ = None

        if param.debug:
            self.feuille.save_log("REQUETE FEUILLE: " + commande)

        #  mettre en toute fin, pour des raisons de scurit.
        if pylib.securite.keywords_interdits_presents(commande):
            self.erreur("Mots-clefs interdits : " + ", ".join(sorted(pylib.securite.keywords_interdits)))
        try:
            code = compile(commande, '<string>', 'eval')
            return str(eval(code, self.feuille.objets))
        except SyntaxError:
            exec(commande, self.feuille.objets)
            return u'Commande excute.'
            self.feuille.historique.archiver()
            self.feuille.canvas.redetecter = True
            if self.feuille.classeur is not None and self.feuille.classeur.parent is not None:
                self.feuille.classeur.parent.rafraichir_titre()


    def parser(self, commande):
        commande = commande.strip()

        while '  ' in commande:
            commande = commande.replace('  ', ' ')

        if commande.startswith("="):
            commande = "_" + commande

        if commande == "del":
            commande += " _"

        # Gestion des '
        # NB: attention, \' a dj un sens en LaTeX
        commande = commande.replace("'", "_prime").replace("\\_prime", "\\'")

        # (A B) -> Droite(A,B)
        def f(m):
            return "Droite(%s, %s)" % m.groups()
        commande = re.sub(r"\([ ]?(" + mathlib.parsers.VAR + ")[ ](" + mathlib.parsers.VAR + r")[ ]?\)", f, commande)

        # [A B] -> Segment(A,B)
        def f(m):
            return "Segment(%s, %s)" % m.groups()
        commande = re.sub(r"\[[ ]?(" + mathlib.parsers.VAR + ")[ ](" + mathlib.parsers.VAR + r")[ ]?\]", f, commande)

        # ||u|| -> u.norme
        def f(m):
            return "%s.norme" % m.groups()
        commande = re.sub(r"\|\|[ ]?(" + mathlib.parsers.VAR + r")[ ]?\|\|", f, commande)

        # ||A>B|| -> (A>B).norme
        def f(m):
            return "(%s>%s).norme" % m.groups()
        commande = re.sub(r"\|\|[ ]?(" + mathlib.parsers.VAR + ")>(" + mathlib.parsers.VAR + r")[ ]?\|\|", f, commande)

        # 1,2 ou 1;2 ou 1 2 ou (1,2) ou (1;2) ou (1 2) *uniquement* -> Point(1,2)
        m = re.match("(\()?(?P<x>" + mathlib.parsers.NBR_SIGNE + ")[ ]?[;, ][ ]?(?P<y>" + mathlib.parsers.NBR_SIGNE + ")(?(1)\))$", commande)
        if m:
            dict = m.groupdict()
            commande = "Point(%s,%s)" % (dict["x"], dict["y"])

        # `Bonjour !` -> Texte("Bonjour !")
        # NB: attention, \` a dj un sens en LaTeX
        def f(m):
            return "Texte(\"%s\")" % m.groups()[0]
        commande = re.sub(r"(?<!\\)`(([^`]|\\`)*[^`\\]|)`", f, commande)
        return commande





class Historique_feuille(object):
    u"""Historique de la feuille.
    Permet d'enregistrer l'tat de la feuille  un instant donn, et de le restaurer ensuite."""

    def __init__(self, feuille):
        self.__feuille__ = feuille
        self.n = param.nbr_annulations
        self.etats = []
        self.archiver()

##    def archiver(self, commande = ""):
##        # avec la molette de la souris, on effectue une succession rapide de zooms
##        # pour eviter que ca ne remplisse l'historique, on archive alors l'etat actuel a la place du precedent.
##        if commande.startswith("zoomer(") and self.derniere_commande.startswith("zoomer("):
##            self.etats[-1] = self.__feuille__.sauvegarder()#False)
##        else:
##            self.etats.append(self.__feuille__.sauvegarder())#False))
##        self.etats_annules = []
##        if len(self.etats) > self.n:
##            self.etats.pop(0) # plus rapide que "self.etats = self.etats[-self.n:]"
##        self.derniere_commande = commande

    def archiver(self, signature = None):
        sauvegarde = self.__feuille__.sauvegarder()
        # On vite de stocker deux fois de suite la mme chose dans l'historique.
        if self.etats and sauvegarde == self.etats[-1]:
            return
        # Avec la molette de la souris, on effectue une succession rapide de zooms.
        # Pour viter que a ne remplisse l'historique, on archive alors l'tat actuel
        #  la place du prcedent. Ceci s'effectue grce  la signature.
        # De manire gnrale, si signature != None, lorsque deux demandes d'archivages
        # successives parviennent avec la mme signature, la seconde crase la premire.
        if signature is not None and self._derniere_signature == signature:
            self.etats[-1] = sauvegarde
        else:
            self.etats.append(sauvegarde)
        self.etats_annules = []
        if len(self.etats) > self.n:
            self.etats.pop(0) # plus rapide que "self.etats = self.etats[-self.n:]"
        self._derniere_signature = signature

    def annuler(self):
        if len(self.etats) > 1:
            etat_actuel = self.etats.pop()
            self.etats_annules.append(etat_actuel)
            if len(self.etats_annules) > self.n:
                self.etats_annules.pop(0) # plus rapide que "self.etats_annules = self.etats_annules[-self.n:]"
            self.restaurer(self.etats[-1])
            self.__feuille__.message(u"Action annule.")
        else:
            self.__feuille__.message(u"Impossible d'annuler.")



    def refaire(self):
        if self.etats_annules:
            etat = self.etats_annules.pop()
            self.etats.append(etat)
            if len(self.etats) > self.n:
                self.etats.pop(0) # plus rapide que "self.etats = self.etats[-self.n:]"
            self.restaurer(etat)
            self.__feuille__.message(u"Action restaure.")
        else:
            self.__feuille__.message(u"Impossible de restaurer.")



    def restaurer(self, txt):
        self.__feuille__.effacer(False)
        self.__feuille__.charger(txt)










class Feuille(object):
    u"""Feuille de travail.

    L'objet 'log' doit tre une liste destine  contenir tous les messages.
    """

    def __hash__(self):
        return id(self)

    def __init__(self, classeur = None, titre = "", log = None):
        self.log = log
        self.classeur = classeur
        if getattr(self.classeur, "parent", None) is not None:
            self._fenetre =  self.classeur.parent.param("fenetre")
        else:
            self._fenetre = param.fenetre

        self.macros = {}
        self._cache_listes_objets = {}
        self._actualiser_liste_objets = True
        self._objets_temporaires = []
        self.__point_temporaire__ = None
        # Verrou qui indique que des objets sont encore en cours de modification.
        self._verrou_affichage = None
        # On met ._stop  True pour stopper toutes les animations en cours.
        self._stop = False
        self._afficher_objets_caches = False

        self.objets = Dictionnaire_objets(self)
        self.historique = Historique_feuille(self)
        self.interprete = Interprete_feuille(self)

        # Parametres permettant de gerer l'enregistrement:
        self.sauvegarde = {
                    "modifie": True,            # modifications depuis dernire sauvegarde
                    "repertoire": None,         # rpertoire de sauvegarde
                    "nom": None,          # nom de sauvegarde
                    "export": None,        # nom complet utilis pour l'export
                    }

        # Informations sur le document
        self.infos = {
            "titre": titre,
            "auteur": param.utilisateur,
            "creation": time.strftime("%d/%m/%Y - %H:%M:%S",time.localtime()),
            "modification": time.strftime("%d/%m/%Y - %H:%M:%S",time.localtime()),
            "version": "",
            "resume": "",
            "notes": "",
            }


#        Objet.__feuille__ = self # les objets sont crees dans cette feuille par defaut


    def liste_objets(self, objets_caches = None, tri = False):
        u"""Liste des objets, tris ventuellement selon le style 'niveau'.

        NB: un systme de mise en cache est utilis si possible, contrairement  .objets.lister()."""
        if self._actualiser_liste_objets:
            for key in self._cache_listes_objets:
                self._cache_listes_objets[key] = None
        if objets_caches is None:
            objets_caches = self.afficher_objets_caches
        # 4 caches, correspondants aux 4 situations possibles :
        # objets_caches = True, trier = True ;
        # objets_caches = True, trier = False ; etc.
        clef = 'c' if objets_caches else ''
        if tri:
            clef += 't'
        if self._cache_listes_objets.get(clef) is None:
            liste = self.objets.lister(objets_caches = objets_caches)
            if tri:
                liste.sort(key = lambda x:x.style("niveau"), reverse = True)
            self._cache_listes_objets[clef] = liste
        return self._cache_listes_objets[clef]


    @property2
    def afficher_objets_caches(self, valeur = None):
        if valeur is not None:
            self._afficher_objets_caches = valeur
            for objet in self.objets.liste_objets(True):
                if not objet.style('visible'):
                    objet.creer_figure()
            self._actualiser_liste_objets = True
        return self._afficher_objets_caches


#########################################################################################

    @property
    def canvas(self):
        if self.classeur is not None and self.classeur.parent is not None:
            return self.classeur.parent.canvas
        else:
            return _pseudocanvas


    @property
    def nom(self):
        u"Destin  tre affich."
        nom = self.infos["titre"]
        if self.sauvegarde["nom"]:
           nom += ' - ' + self.sauvegarde["nom"]
        return nom or "Feuille"


    def nom_complet(self):
        u"Destin  tre affich en haut de la fentre."
        nom = self.sauvegarde["modifie"] and "* " or ""
        liste = self.sauvegarde["nom"], self.infos["titre"]
        nom += " - ".join(s for s in liste if s)
        return nom



    def objet(self, nom):    # retourne l'objet associe au nom "nom"
        return self.objets[nom]



#######################################################################################

# Methodes se rapportant a la feuille elle-meme


    def redefinir(self, objet, valeur):
        nom = objet.nom
        # on rcupre la liste des arguments, et on la formate...
        args = valeur.strip()[valeur.find("("):-1] + ",)"
        # ...de manire  obtenir un objet 'tuple' en l'valuant.
        args = eval(args, self.objets) # FAILLE POTENTIELLE DE SECURITE ?
        heritiers = objet._heritiers()
        heritiers.add(objet)
        for arg in args:
            if isinstance(arg, Objet):
                for heritier in heritiers:
                    if arg is heritier:
                        self.erreur(u"Dfinition circulaire dans %s : l'objet %s se retrouve dpendre de lui-mme." %(valeur, nom))
                        #raise RuntimeError, "Definition circulaire dans %s : l'objet %s se retrouve dependre de lui-meme." %(valeur, nom)
        actuel = self.sauvegarder()
        if isinstance(objet, Texte):
            nouveau = actuel.replace("\n" + nom + "=" + objet.__repr__(False), "\n" + nom + "=" + valeur)
        else:
            valeur += "\n" + nom + ".copier_style(" + repr(objet) + ")"
            nouveau = actuel.replace("\n" + nom + "=" + repr(objet), "\n" + nom + "=" + valeur)
        print nouveau
        try:
            self.historique.restaurer(nouveau)
        except:
            print_error()
            self.historique.restaurer(actuel)
            self.erreur(u"Erreur lors de la redfinition de %s." %nom)



    def inventaire(self):
        if param.debug:
            for obj in self.liste_objets(True):
                print "- " + obj.nom + " : " + repr(obj) + " (" + obj.type() + ")"
        liste = [unicode2(obj.nom_complet) + u" (" + unicode2(obj.titre(point_final = False))
                + (not obj.style("visible") and u" invisible)" or u")") for obj in self.liste_objets(True)]
        liste.sort()
        return liste

    def effacer_codage(self):
        u"Efface tous les codages sur les segments, angles et arcs de cercles."
        for obj in self.liste_objets(True):
            if obj.style("codage") is not None:
                obj.style(codage = "")
                obj.creer_figure()
        self.canvas.actualiser()


    def coder(self):
        u"Codage automatique de la figure (longueurs gales, angles gaux, et angles droits)."
        def test(groupe, liste, i):
            if len(groupe) is 1:
                groupe[0]["objet"].style(codage = "")
                groupe[0]["objet"].creer_figure()
                return False
            else:
                try:
                    for elt in groupe:
                        elt["objet"].style(codage = liste[i])
                        elt["objet"].creer_figure()
                    return True
                except IndexError:
                    self.message(u"Le nombre de codages disponibles est insuffisant.")
                    print_error(u"Le nombre de codages disponibles est insuffisant.")

        lignes = [{"longueur": obj._longueur(), "objet": obj} for obj in self.objets.lister(False, type = (Segment, Arc_generique))]
        if lignes:
            lignes.sort() # attention, le classement d'un dictionnaire se fait selon l'ordre alphabtique des clefs
            groupe = [lignes[0]]
            i = 1
            for ligne in lignes[1:]:
                if abs(groupe[-1]["longueur"] - ligne["longueur"]) < contexte['tolerance']:
                    groupe.append(ligne)
                else:
                    resultat = test(groupe, param.codage_des_lignes, i)
                    if resultat is None:
                        break
                    if resultat:
                        i += 1
                    groupe = [ligne]
            test(groupe, param.codage_des_lignes, i)


        angles = [{"angle": obj.val, "objet": obj} for obj in self.objets.lister(False, type = Secteur_angulaire)]
        if angles:
            angles.sort() # attention, le classement d'un dictionnaire se fait selon l'ordre alphabtique des clefs
            groupe = [angles[0]]
            i = 2
            for angle in angles[1:]:
                if abs(groupe[-1]["angle"] - angle["angle"]) < contexte['tolerance']:
                    groupe.append(angle)
                else:
#                    print abs(abs(groupe[-1]["angle"]) - math.pi/2)
                    if abs(abs(groupe[-1]["angle"]) - math.pi/2) < contexte['tolerance']:
                        for elt in groupe:
                            elt["objet"].style(codage = "^")
                            elt["objet"].creer_figure()
                    else:
                        resultat = test(groupe, param.codage_des_angles, i)
                        if resultat is None:
                            break
                        if resultat:
                            i += 1
                    groupe = [angle]
            if abs(abs(groupe[-1]["angle"]) - math.pi/2) < contexte['tolerance']:
                for elt in groupe:
                    elt["objet"].style(codage = "^")
                    elt["objet"].creer_figure()
            else:
                test(groupe, param.codage_des_angles, i)

        self.canvas.actualiser()

    def relier_axe_x(self, point):
        if isinstance(point, Point_generique):
            with self.canvas.geler_affichage(actualiser = True):
                M = Point("%s.x" %point.nom, 0, fixe = True)
                M.label("${%s.x}$" %point.nom, formule = True)
                s = Segment(point, M, style = ":")
                self.objets.add(M)
                self.objets.add(s)

    def relier_axe_y(self, point):
        if isinstance(point, Point_generique):
            with self.canvas.geler_affichage(actualiser = True):
                M = Point(0, "%s.y" %point.nom, fixe = True)
                M.label("${%s.y}$" %point.nom, formule = True)
                s = Segment(point, M, style = ":")
                self.objets.add(M)
                self.objets.add(s)


    def relier_axes(self, point):
        with self.canvas.geler_affichage(actualiser = True):
            self.relier_axe_x(point)
            self.relier_axe_y(point)
    #def ajouter_objets_temporaires(self, *objets):
    #    self.objets.__dict__["_objets_temporaires"] += list(objets)

    #def effacer_objets_temporaires(self):
    #    self.objets.__dict__["_objets_temporaires"] = []

    def objet_temporaire(self, objet = False):
        if objet is not False:
            if self._objets_temporaires:
                obj = self._objets_temporaires[0]
                self.canvas.actualiser()
            if objet is None:
                self._objets_temporaires = []
            else:
                objet.__feuille__ = self
                self._objets_temporaires = [objet]
        return self._objets_temporaires


    def contient_objet(self, objet):
        """contient_objet(self, objet) -> bool

        Teste rapidement si l'objet est rpertori dans la feuille.
        (Ne cherche pas parmi les objets temporaires.)"""

        return is_in(objet, self.objets.values())


    def contient_objet_temporaire(self, objet):
        """contient_objet_temporaire(self, objet) -> bool

        Teste rapidement si l'objet est rpertori comme objet temporaire dans la feuille."""

        for obj in self._objets_temporaires:
            if obj is objet:
                return True
        return False


    def point_temporaire(self):
        if self.__point_temporaire__ is None:
            self.__point_temporaire__ = Point()
            self.__point_temporaire__.__feuille__ = self
        return self.__point_temporaire__

    def start(self):
        u"Autorise le lancement d'animations."
        self._stop = False

    def stop(self):
        u"Arrte toutes les animations en cours."
        self._stop = True

    def animer(self, nom, debut = 0, fin = 1, pas = 0.02, periode = 0.03):
        u"Anime la variable nomme 'nom'."
        self.objets[nom].varier(debut, fin, pas, periode)

#######################################################################################

# Gestion des message

    def message(self, messg): # A REECRIRE
        if self.canvas:
            messg = "Feuille %s - %s" %(self.nom, messg)
            self.canvas.message(messg)
        elif param.afficher_messages:
            print(messg)
            if self.log is not None:
                self.log.append(message)


    def erreur(self, message, erreur = None):
        self.message(u"Erreur : " + unicode2(message))
        if erreur is None:
            erreur = RuntimeError
        raise erreur, str3(message)


    def save_log(self, log):
        # Imprativement utiliser 'is not None' car le log peut tre vide.
        if self.log is not None:
            self.log.append(log)


#######################################################################################

# Gestion de l'affichage

#    cf. API/affichage.py


    def lister_figures(self, objet_deplace = None):
        if objet_deplace is None:
            heritiers = []
        else:
            heritiers = objet_deplace._heritiers()
            heritiers.add(objet_deplace)
        # objets non susceptibles d'tre modifis (sauf changement de fentre, etc.)
        liste1 = []
        # objets susceptibles d'tre modifis
        liste2 = []
        for objet in self.liste_objets():
            liste = liste2 if is_in(objet, heritiers) else liste1
            liste.extend(objet._representation)
            liste.extend(objet._trace)
            if objet.etiquette is not None:
                liste.extend(objet.etiquette._representation)
                liste.extend(objet.etiquette._trace)
        for objet in self._objets_temporaires:
            liste2.extend(objet._representation)
            liste2.extend(objet._trace)
            if objet.etiquette is not None:
                liste2.extend(objet.etiquette._representation)
                liste2.extend(objet.etiquette._trace)
        return liste1, liste2


    def rafraichir_figures(self, tous_les_objets = False):
        u"""Recre les figures des objets sensibles  la fentre d'affichage.

        Si tous_les_objets = True, les figure des objets non sensibles sont
        aussi rcres.

        En principe, cette mthode n'a pas  tre appele directement, le
        canevas s'en charge.
        """
        for objet in self.liste_objets():
            # Il faut recalculer la position de toutes les tiquettes
            # quand la fentre d'affichage change.
            if objet.etiquette:
                objet.etiquette._cache.clear()
            if objet._affichage_depend_de_la_fenetre:
                try:
                    objet.creer_figure()
                except:
                    self.message(u"Erreur lors de l'affichage de %s." % objet.nom)
                    print_error(u"Erreur lors de l'affichage de l'objet " + objet.nom)
            else:
                # Mme si tous les objets n'ont pas besoin d'tre rafraichis,
                # leurs tiquettes doivent l'tre
                if objet.etiquette:
                    objet.etiquette.creer_figure()




    def effacer_traces(self):
        u"Efface toutes les traces (sans enlever le mode trace des objets)."
        for objet in self.liste_objets():
            objet.effacer_trace()
        self.canvas.actualiser()

    def objets_en_gras(self, *objets):
        u"""Met en gras les objets indiqus, et remet les autres objets en tat "normal" le cas chant."""

        changements = False
        for objet in self.liste_objets(True):
            if is_in(objet, objets):
                val = objet.en_gras(True)
            else:
                val = objet.en_gras(False)
            if val is not None:
                changements = True

        if changements:
            self.canvas.actualiser()
#########################################################################################



    def sauvegarder(self):
        u"Renvoie l'ensemble des commandes python qui permettra de recrer la figure avec tous ses objets."

        objets = self.liste_objets(True)
        # on doit enregistrer les objets dans le bon ordre (suivant la _hierarchie) :
        objets.sort(key=lambda x:getattr(x,"_hierarchie"))

        texte = "fenetre = " + repr(self.fenetre) + "\n"
        a_rajouter_a_la_fin = ""

        for objet in objets:
            if isinstance(objet, Texte) and objet.style("legende") == wxgeo.FORMULE:
                # on fait un cas particulier pour les objets Texte, car ils peuvent contenir une formule
                # qui dpend d'autres objets. Leur style n'est alors appliqu qu'aprs.
                texte += objet.nom + "=" + objet.__repr__(False) + "\n"
                a_rajouter_a_la_fin += objet.nom + ".style(**" + repr(objet.style()) + ")\n"
            else:
                texte += objet.nom + "=" + repr(objet) + "\n"

        # Les tiquettes peuvent contenir des formules qui dpendent d'autres objets.
        for objet in objets:
            if objet.etiquette is not None:
                texte += objet.nom + ".etiquette.style(**" + repr(objet.etiquette.style()) + ")\n"

        return texte + a_rajouter_a_la_fin




    def effacer(self, rafraichir = True):
        self.objets._Dictionnaire_objets__effacer()
        if rafraichir and self.canvas is not None:
            self.canvas.actualiser()




    def charger(self, commandes, rafraichir = True):
        u"""Excute un ensemble de commandes dans la feuille.

        Usage:
        f = Feuille()
        ...
        commandes = f.sauvegarder()
        f.effacer()
        f.charger(commandes)
        """
        with self.canvas.geler_affichage(actualiser = rafraichir):
            try:
                exec(commandes, self.objets)
            except:
                print "Liste des commandes:"
                print "--------------------"
                print commandes
                print "--------------------"
                try:
                    print_error()
                except:
                    print u"Affichage de l'erreur impossible !"
                self.erreur(u"Chargement incomplet de la feuille.")


    def executer(self, commande, parser = True):
        return self.interprete.executer(commande, parser = parser)


#########################################################################################

# Gestion du zoom et des coordonnees, reglage de la fenetre d'affichage.

#    cf. API/affichage.py

    def reinitialiser_fenetre(self):
        self._fenetre = param.fenetre
        self.canvas.redessiner_et_actualiser()

    def _get_fenetre(self):
        return self._fenetre

    def _set_fenetre(self, xmin_xmax_ymin_ymax):
        xmin, xmax, ymin, ymax =xmin_xmax_ymin_ymax

        if xmin is None:
            xmin = self._fenetre[0]
        if xmax is None:
            xmax = self._fenetre[1]
        if ymin is None:
            ymin = self._fenetre[2]
        if ymax is None:
            ymax = self._fenetre[3]
        if abs(xmax - xmin) < 10*contexte['tolerance'] or abs(ymax - ymin) < 10*contexte['tolerance']:
            self.erreur("Le rglage de la fentre est incorrect.", ValueError)
        # Les 'float()' servent  contourner un bug de numpy 1.1.x et numpy 1.2.x (repr de float64)
        self._fenetre = float(min(xmin, xmax)), float(max(xmin, xmax)), float(min(ymin, ymax)), float(max(ymin, ymax))
        self.canvas.redessiner_et_actualiser()

    fenetre = property(_get_fenetre, _set_fenetre, reinitialiser_fenetre)



#########################################################################################

# Diverses fonctionnalites de la feuille, utilisees par les objets.
# C'est un peu la boite a outils :-)




    def nom_aleatoire(self, prefixe = ALL.Objet._prefixe_nom): # utilise pour la generation d'objets de noms toujours differents
        #print self.objets.__noms__()
        liste = [int(s[len(prefixe):]) for s in self.objets.noms if re.match(prefixe + "[0-9]+$", s)]
        #print liste
        return prefixe + str(max(liste or [0]) + 1)



    def pause(self):
        souffler()
        if self._stop:
            raise RuntimeError, "Interruption de la macro."
