# -*- coding: iso-8859-1 -*-
from __future__ import division # 1/2 == .5 (par defaut, 1/2 == 0)

##--------------------------------------#######
#                   Fonctions                    #
##--------------------------------------#######
#    WxGeometrie
#    Dynamic geometry, graph plotter, and more for french mathematic teachers.
#    Copyright (C) 2005-2007  Nicolas Pourcelot
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

# version unicode


from objet import *



class Fonction(Objet_numerique):
    u"""Une fontion.

    Une fonction numrique  une variable; l'argument est une expression sous forme de chaine de caractres.
    Exemple: Fonction('2*x+1', variable = 'x', ensemble = 'R')."""

    _prefixe_nom = "f"

    # nom de variable (mais pas d'attribut)
    __re = re.compile('(' + parsers.VAR_NOT_ATTR + ')')


    __expression = Argument("basestring")
    __ensemble = Argument("basestring")
    variable = __variable = ArgumentNonModifiable("basestring")

    @property2
    def expression(self, expression = None):
        if expression is None:
            if self.__liste_expression:
                # On regnre l'expression  partir de l'expression compile.
                # C'est important, car certains objets de la feuille peuvent avoir chang de nom entre temps.
                expr = ""
                for elt in self.__liste_expression:
                    if isinstance(elt, Objet):
                        expr += elt.nom
                    else:
                        expr += elt
                return expr
            else:
                return self.__expression
        else:
            self.modifier_expression_et_ensemble(expression = expression)


    @property2
    def ensemble(self, ensemble = None):
        if ensemble is None:
            if self.__liste_ensemble:
                # On regnre l'ensemble  partir de l'ensemble compile.
                # C'est important, car certains objets de la feuille peuvent avoir chang de nom entre temps.
                ens = ""
                for elt in self.__liste_ensemble:
                    if isinstance(elt, Objet):
                        ens += elt.nom
                    else:
                        ens += elt
                return ens
            else:
                return self.__ensemble
        else:
            self.modifier_expression_et_ensemble(ensemble = ensemble)


    def modifier_expression_et_ensemble(self, expression = None, ensemble = None):
        u"""Si l'on modifie  la fois l'expression et l'ensemble,
        il est beaucoup plus rapide d'utiliser cette mthode."""
        print "modification!"
        if expression is None:
            expression = self.__expression
        if ensemble is None:
            ensemble = self.__ensemble
        else:
            ensemble = intervalles.preformatage_ensemble(ensemble) or "]-oo;+oo["
        if ensemble == self.__ensemble and expression == self.__expression:
            # Aucune modification
            return
        args = self._test_dependance_circulaire(expression, ensemble)
        self.__expression = expression
        self.__ensemble = ensemble
        self._compile(*args)


    def __init__(self, expression, ensemble = "R", variable = "x", **styles):
        Objet.__init__(self,  **styles)
        # *Pr*formatage
        ensemble = intervalles.preformatage_ensemble(ensemble) or "]-oo;+oo["
        self.__liste_expression = []
        self.__liste_ensemble = []
        # Une fonction peut tre dfinie par morceaux
        # Liste des fonctions correspondant  chaque morceau
        self.__fonctions = None
        # Liste des (unions d')intervalles correspondant  chaque morceau
        self.__unions = None
        # Les arguments non modifiables ne sont pas encapsuls dans des rfrences (classe Ref)
        self.__expression = expression = Ref(expression)
        self.__ensemble = ensemble = Ref(ensemble)
        self.__variable = variable


    def _test_dependance_circulaire(self, expression, ensemble):
        u"""Provoque une erreur si l'objet se retrouve dpendre de lui-mme avec la nouvelle valeur.

        Retourne deux listes (list) composes alternativement d'instructions et d'objets de la feuille,
        et un ensemble (set) constitu des objets de la feuille mis en jeu dans le code.
        (...  documenter ...)"""
        if self.__feuille__ is not None:
            liste_expression = re.split(self.__re, expression)
            liste_ensemble = re.split(self.__re, ensemble)
            objets = set()
            for i in xrange(1, len(liste_expression), 2):
                obj = self.__feuille__.objets[liste_expression[i]]
                if isinstance(obj, Objet):
                    liste[i] = obj
                    objets.add(obj)
                    if self is obj or is_in(self, obj._tous_les_ancetres()):
                        print self,
                        raise RuntimeError, "Definition circulaire dans %s : l'objet %s se retrouve dependre de lui-meme." %(self, obj)
            for i in xrange(1, len(liste_ensemble), 2):
                obj = self.__feuille__.objets[liste_ensemble[i]]
                if isinstance(obj, Objet):
                    liste[i] = obj
                    objets.add(obj)
                    if self is obj or is_in(self, obj._tous_les_ancetres()):
                        print self,
                        raise RuntimeError, "Definition circulaire dans %s : l'objet %s se retrouve dependre de lui-meme." %(self, obj)
            return liste_expression, liste_ensemble, objets
        return None, None, None


    def _compile(self,  liste_expression, liste_ensemble, objets):
        print "compilation !"
        u"""Compile l'expression stocke dans la variable ; les arguments sont les valeurs retournes par '_test_dependance_circulaire'.

        La compilation doit toujours avoir lieu  la fin de la procdure de redfinition de la variable,
        car elle ne doit tre excute que si la redfinition de la variable va effectivement avoir lieu,
        c'est--dire si tout le processus prcdent s'est excut sans erreur."""
        if self.__feuille__ is not None:
            self.__liste_expression = liste_expression
            self.__liste_ensemble = liste_ensemble
            self.__fonctions = []
            self.__unions = []
            expressions = self.__expression.split("|")
            ensembles = self.__ensemble.split("|")
            # TODO: Prvoir le cas o les deux listes ne sont pas de mme longueur
            n = min(len(expressions), len(ensembles))
            for i in xrange(n):
                express = parsers.traduire_formule(  expressions[i], \
                                                                    fonctions = [key for key, obj in self.__feuille__.objets.items() if hasattr(obj, "__call__")]
                                                                    )
                # On force ensuite la variable  apparaitre dans l'expression de la formule.
                # C'est important quand la fonction est constante :
                # l'image d'un tableau par la fonction doit tre un tableau, et non la constante.
                if self.__variable not in express:
                    express += "+0.*" + self.__variable
                self.__fonctions.append(eval("lambda " + self.__variable + ":" + express, self.__feuille__.objets))
                ensemb = intervalles.formatage_ensemble(ensembles[i], preformatage = False)
                self.__unions.append(eval(ensemb, self.__feuille__.objets))

            # on supprime la variable de la liste des vassaux pour les objets dont elle ne dpendra plus desormais:
            for objet in self._ancetres:
                objet.vassaux.remove(self)
            self._ancetres = objets
            self._modifier_hierarchie()
            for objet in self._ancetres:   # l'objet est vassal de chacun des objets dont il depend
                objet.vassaux.append(self)
        else:
            for objet in self._ancetres:
                objet.vassaux.remove(self)
            self._ancetres = set()
            self._modifier_hierarchie()
            self.__liste_expression = []
            self.__liste_ensemble = []
            self.__fonctions = None
            self.__unions = None


    def _set_feuille(self):
        self._compile(*self._test_dependance_circulaire(self.__expression, self.__ensemble))
        self._heritiers_a_recalculer(self._heritiers())




    def _recenser_les_ancetres(self):
#        warning("'_recenser_les_ancetres' n'a aucun effet pour une variable.")
        self._modifier_hierarchie()






    def affiche(self, actualiser = False):
        if actualiser and self.__feuille__ is not None:
            canvas = self.__canvas__
            if canvas:
                canvas.actualiser()


    @staticmethod
    def _convertir(objet):
        u"Convertit un objet en fonction."
        return ALL.Fonction(objet)


    def __call__(self, valeur):
        for i in xrange(len(self.__unions)):
            if valeur in self.__unions[i]:
                return self.__fonctions[i](valeur)
        raise ValueError, "math domain error"

