# -*- coding: iso-8859-1 -*-

import math

#------------------------------------------------------------------------------
def determinant(a, b, c):
  """
  determinant(a, b, c)
  
  Retourne le dterminant de l'quation ax + bx + c = 0
  det = b-4ac
  """
  return b**2-4*a*c

#------------------------------------------------------------------------------
def quadratic(a, b, c):
  """
  quadratic(a, b, c)
  
  Retourne les solutions de l'quation du second degr ax + bx + c = 0
  
  Si le dterminant est positif ou nul, on a deux racines relles :
  x1 = (-b + sqrt(b - 4ac)) / (2a)
  x2 = (-b - sqrt(b - 4ac)) / (2a)
  
  Si le dterminant est ngatif, on a deux racines complexes conjugues :
  x1 = (-b + sqrt(abs(b - 4ac)) * j) / (2a)
  x2 = (-b - sqrt(abs(b - 4ac)) * j) / (2a)
  
  Exemple :
  quadratic(1,5,-2) =>  (-5.3722813232690143, 0.37228132326901431) 
  quadratic(1,-4,13) => ((2-3j), (2+3j))
  """
  D = determinant(a, b, c)
  x1 = None
  x2 = None
  if a==0:
    x1 = -c/b
  elif D>=0:
    x1 = (-b + math.sqrt(D))/(2*a)
    if D>1e-16:
      x2 = (-b - math.sqrt(D))/(2*a)
      x1, x2 = min(x1, x2), max(x1, x2)
  else:
    x1 = complex(-b/(2*a), -math.sqrt(abs(D))/(2*a))
    if abs(D)>1e-16:
      x2 = complex(-b/(2*a), math.sqrt(abs(D))/(2*a))
  return x1, x2

#------------------------------------------------------------------------------
def pyth(a, b):
  """
  pyth(a, b)

  Retourne la dimension d'un cot du triangle. Les paramtres attendus sont :
  l'hypothnuse et un cot. Ces paramtres peuvent tre passs dans n'importe
  quel ordre (SciCalc considrera qu l'hypothuse est la plus grande dimension
  fournie). Les valeurs peuvent tre ngatives, les calculs seront effectus sur
  leurs valeurs absolues.
  pyth = sqrt(a - b)
  """
  a, b = abs(a), abs(b)
  a, b = max(a,b), min(a,b)
  return math.sqrt(a**2 - b**2)

#------------------------------------------------------------------------------
def prod(facteurs):
  return reduce(lambda x,y:x*y, facteurs, 1)

#------------------------------------------------------------------------------
def factoriel(n, maximum = 20000):
  """
  fact(n)
  
  Retourne la factorielle (n!) de n.
  n! = n*(n-1)*(n-2)*...*1
  0! = 1
  
  Exemple :
  factoriel(10) => 3628800
  """
  if n > maximum:
    raise OverflowError
  return prod(range(1, n + 1))

#------------------------------------------------------------------------------
def gcd(a, b):
  """
  pgcd(a,b)
  
  Retourne le pgcd (plus grand commun diviseur) de a et b
  
  Exemple :
  pgcd(12,18) => 6
  """
  a, b = max(abs(a),abs(b)), min(abs(a),abs(b))
  while b:
    a, b = b, a%b
  return a

#------------------------------------------------------------------------------
def lcm(a, b):
  """
  ppcm(a,b)
  
  Retourne le ppcm (plus petit commun multiple) de a et b
  
  Exemple :
  ppcm(12,18) => 36
  """
  return a*b/gcd(a,b)

#------------------------------------------------------------------------------
def pgcd(*termes):
  """
  pgcd(a,b)
  
  Retourne le pgcd (plus grand commun diviseur) de a et b
  
  Exemple :
  pgcd(12,18) => 6
  """
  return reduce(lambda x,y:gcd(x,y), termes)

#------------------------------------------------------------------------------
def ppcm(*termes):
  """
  ppcm(a,b)
  
  Retourne le ppcm (plus petit commun multiple) de a et b
  
  Exemple :
  ppcm(12,18) => 36
  """
  return reduce(lambda x,y:lcm(x,y), termes)

#------------------------------------------------------------------------------
def factor(n):
  """
  factor(n), alias facteur()
  
  Dcompose n en produit de facteurs premiers
  
  Exemple :
  factor(35) -> [5, 7]
  """
  p = [] # liste des facteurs
  if n < 0:
    n = abs(n)
    p.append(-1)
  
  while n%2 == 0:
    n = n/2; p.append(2)
  
  i = 3
  m = math.sqrt(n)
  while i <= m:
    while n%i == 0:
      n = n/i; m = m/math.sqrt(i)
      p.append(i)
    i += 2
      
  if n <> 1:
    p.append(n)
  
  return p    

#------------------------------------------------------------------------------
def facteur(n):
  return factor(n)

#------------------------------------------------------------------------------
def diviseur(n):
  """
  diviseur(n)
  
  retourne la liste des diviseurs de l'entier pass en paramtre
  
  Exemple :
  diviseur(12) -> [1, 2, 3, 4, 6, 12]
  """
  p = [] # liste des diviseurs
  if n < 0:
    n = abs(n)
    p.append(-1)
  
  i = 1
  m = math.sqrt(n)
  while i <= m:
    if n%i == 0:
      p += [n/i, i]
    i += 1
    #print i
  
  p.sort()
  return p
   
#------------------------------------------------------------------------------
def sqr(a):
  """
  sqr(a)
  
  Retourne le carr de a
  """
  return a**2

#------------------------------------------------------------------------------
def ln(x):
  """
  ln(x)
  
  Retourne le logarithme en base e de x (logarithme nprien)
  """
  return math.log(x)

#------------------------------------------------------------------------------
def logn(x, n):
  """
  logn(x, n)
  
  Retourne le logarithme en base n de x
  """
  return math.log10(x)/math.log10(n)
